# logische Operatoren

*Dateiname: boolescheAusdrucke.md*

?>Du kennst die Operatoren UND, ODER, Exklusives ODER und die NEGIERUNG.<br />
Du kannst mit diesen Operatoren verschiedene Bedingungen verknüpfen.

## boolesche Ausdrücke

Logische Operatoren vereinfachen eine Vielzahl von geschachtelten Verzweigungen - eben mit ein bisschen Logik.
Denke an das Einführungsbeispiel der Verzweigung - den springenden Ball. Dort war es nötig folgenden Quellcode zu schreiben:

```
int speed = 1;
…
void draw() {
  background(255);
  ellipse(x, 50, 20, 20);
  x = x + speed;
  if (x>width) {
    speed = -speed;
  }
  if (x<0) {
    speed = -speed;
  }

}
```

Dies kann natürlich schöner (mit Hilfe einer
ODER-Verknüpfung) „gecoded“ werden:

```
…
  x = x + speed;
  if (x>width || x<0) {
    speed = -speed;
  }
…
```

Für solche Fälle stehen dir folgende logische Operatoren zur Verfügung:

### UND-Verknüpfung

a | b | a && b
-- | -- | --
false | false | false
false | true | false
true | false | false
true | true | true

**Beispiel**

```
int k = 5;
float f = 2.5;
int t = 30;
 
if (k > 0 && f < 10.0 || t > 100) {
  println("hurra");
} 
```

### ODER-Verknüpfung

a | b | a ODER B
-- | -- | --
false | false | false
false | true | true
true | false | true
true | true | true

d>Das ODER wird als **||** dargestellt. Die Markdownformatierung widerspricht dem jedoch, da | zur Spaltensetzung genutzt wird.

s>**Und wo ist dieses Zeichen | auf meiner Tastatur...?**<br />Auf dem Mac mit ALT+7, unter Windows mit Alt Gr + <.

**Beispiel**

```
void draw() {
  background(255);
  ellipse(x, 50, 20, 20);
  x = x + speed;
 
  if (x < 0 || x > width) {
      speed = -speed;
  }
}
```

### XOR-Verknüpfung

a | b | a ^ b
-- | -- | --
false | false | false
false | true | true
true | false | true
true | true | false


### NOT-Verknüpfung

a | !a
-- | --
true | false
false | true

**Beispiel**

```
int x = 5;
if (! (x < 0 || x >= 1024)) {
   println("Bin auf dem Schirm!");
}
```

## Verknüpfung mehrere Aussagen

Betrachten wir dieses Gebiet am Beispiel:

```
int k = 5;
float f = 2.5;
int t = 30;

if (k > 0 && f < 10.0 || t > 100) {
  println("hurra");
}
```

Processing arbeitet in folgender **Abfolge**:
1. Variablenwerte werden eingesetzt
2. einzelne Vergleiche werden aufgelöst
3. Befehle werden von links nach rechts aufgelöst

**Bezug Beispiel** 

Das Ergebnis wäre `true`bzw. "hurra" in der Konsole.

## Anwendungsbeispiel

Unser allseits freudebringender fliegender Ball soll nun möglichst innerhalb des Fensters springen.

Dies kann man wie folgt lösen:

```
int x = 0;
int speed = 1;

void draw() {
  background(255);
  ellipse(x, 50, 20, 20);
  x = x + speed;

  if (x > width) {
    speed = -speed;
  }

  if (x < 0) {
    speed = -speed;
  }
}
```

Deutlich eleganter ist die Lösung mit logischen Operatoren:

```
int x = 0;
int speed = 3;

void draw() {
  background(255);
  ellipse(x, 50, 20, 20);
  x = x + speed;

  if (x < 0 || x > width) {
      speed = -speed;
  }
}
```

## Übung

t>**Aufgabe Dreiecksungleichung**<br />Überprüfe durch die drei vom Nutzer eingegebenen Seitenlängen die Dreiecksungleichung. Weiter soll überprüft werden, ob es sich um ein gleichseitiges oder gleichschenkliges Dreieck handelt. 

t>**Aufgabe Rote Zone**<br />Verwende folgenden Basiscode und verändere ihn so, dass der Ball in der markierten Zone rot wird.

```
int x = 0;

void draw() {
  background(0);

  // Linien zum Markieren der Zone
  stroke(255);
  line(25, 0, 25, height);
  line(75, 0, 75, height);

  // hier wird animiert...
  ellipse(x, 50, 20, 20);
  x++;
  if (x > width) {
    x = 0;
  }
}
```

*Tipp: Überlege genau, was du genau im  `If` änderst: Es geht nur um die Farbe, rot oder weiß.*

t>**Aufgabe Rote Zone II**<br />Verändere den Code aus der obigen Aufgabe so, dass der Ball **außerhalb** der markierten Zone rot wird.

*Wichtig: Löse die Aufgabe mit nur einer **Bedingung**.*

t>**Aufgabe pulsierender Kreis**<br />Ein Kreis mit Durchmesser 50 wird stetig größer, bis er den Rand des Fensters erreicht, dann verkleinert er sich bis Durchmesser 50 usw.

*Hinweis: Achte darauf, dass der Durchmesser nicht negativ wird. Gib ihn dir gegebenenfalls in der Konsole aus.*

t>**Aufgabe pulsierende Farbe**<br />Ein schwarzer Kreis wird stetig weißer bis er ganz weiß ist, dann wieder schwärzer, bis er ganz schwarz ist usw. Auch hier ist das Schema wie in der vorigen Aufgabe: ein veränderlicher Wert (Grauwert) und eine Geschwindigkeit, die sich "umdreht". 

