# Boolesche Variable

*Dateiname: boolescheVariable.md*

Mit der Verzweigung (If-Anweisung) bist jetzt bestens vertraut. Sogar boolesche Ausdrücke kannst du umsetzen.

i>Boolesche Variablen sind prima Zeitgenossen, da sie lediglich einen von zwei möglichen Werten speichern können: `true`oder `false`.

### Wozu braucht man sowas?

Wenn du zum Beispiel eine Animation gestaltest, möchtest du dir merken, ob sie gerade laufen soll (play) oder anhalten muss (pause).

Nehmen wir mal eine einfach Animation:

```
int x = 0;

void draw() {
  background(255);
  ellipse(x, 50, 20, 20);
  x++;

  // immer wieder von links reinkommen
  if (x >= 100) {
    x = 0;
  }
}
```

Um einen einfachen play/pause Mechanismus zu implementieren, führen wir eine boolesche Variable `play` ein. Wenn diese `true` ist, soll die Animation laufen. Wenn sie `false` ist, nicht:

```
int x = 0;
boolean play = true;

void draw() {
  background(255);
  ellipse(x, 50, 20, 20);

  // NEU: bewege dich nur, wenn play true ist
  if (play) {
    x++;
  }

  // immer wieder von links reinkommen
  if (x >= 100) {
    x = 0;
  }
}
```

Viel geändert hat sich nicht, aber du kannst mit die Variable play von Hand auf false setzen, dann bleibt der Ball links stehen.

Jetzt müssen Sie nur noch "zur Laufzeit", also während das Programm läuft, die Variable play umsetzen. Das funtioniert, indem Sie die folgende Funktion hinzufügen:

```
void keyPressed() {
  if (play) {
    play = false;
  } else {
    play = true;
  }
}
```

Du setzt also die Variable play in Abhänigigkeit von ihrem aktuellen Zustand, nämlich auf ihr "Gegenteil". In der Logik nennt man das auch die Negation ihres Zustandes. Das kann man ganz elegant formulieren:

```
void keyPressed() {
  play = !play; // setze play auf seine eigene Negation
}
```

### Boolesche Variablen und boolesche Ausdrücke

In eine boolesche Variable kann ein beliebiger boolescher Ausdruck per Zuweisung hineingetan werden. Im folgenden Beispiel wird der Ball in der x-Zone 30-70 rot gefärbt:

```
int x = 0;
boolean inTheZone = false;

void draw() {
  background(255);

  // boolescher Ausdruck
  inTheZone = x > 30 && x < 70;

  if (inTheZone) {
    fill(255, 0, 0); // rot
  }
  else {
    fill(255); // weiß
  }

  ellipse(x, 50, 20, 20);
  x++;
}
```

Auch hier zeigt sich der Vorteil der Varible erst, wenn die Information an anderer Stelle bzw. zu anderer Zeit verwendet wird. Willst du etwa per Tastendruck den Ball wieder auf x=0 setzen **unter der Bedingung**, dass der Ball in 'der Zone' ist, dann fügst du hinzu:

```
void keyPressed() {
  if (inTheZone) {
    x = 0;
  }
}
```

## Übung

t>**Aufgabe Now you see me...**<br />Schreiben Sie ein Programm, dass einen schwarz gefüllten Kreis malt. Wenn man auf die Maustaste drückt, verschwindet er. Beim nächsten Klick erscheint er usw.

*Tipp: Verwende eine boolesche Variable, um dir zu merken, ob dein Kreis sichtbar ist.*

t>**Aufgabe Kreis oder Box**<br />Wo immer der Mauszeiger ist, wird ein weiß gefüllter Kreis gezeichnet (Durchmesser 30). Wenn du auf die Maustaste drückst, wird stattdessen ein schwarz gefülltes Quadrat gezeichnet (Kantenlänge 20). Wenn du wieder die Taste drückst, kommen wieder Kreise usw.

t>**Aufgabe horizontal oder vertikal**<br />Ein Kreis fliegt von links nach rechts und erscheint wieder am linken Rand, wenn er rechts rausfliegt. Wenn man die Maustaste drückt, fliegt der Kreis nach unten und erscheint wieder oben, wenn er den unteren Rand erreicht. Beim nächsten Klick fliegt er wieder horizontal usw.

## Zusammenfassung

w>Boolesche Variablen können einen binären Zustand speichern, d.h. eine Variable enthält entweder den Wert true oder den Wert false. Eine boolesche Variable kann z.B. in einer If-Bedingung verwendet werden.<br /><br />Ein Anwendungsbeispiel ist der Unterschied zwischen "Play" und "Pause" bei einer Animation. Dies kann in einer booleschen Variable play gespeichert werden. Hier würde true bedeuten "Objekt bewegt sich" und false "Objekt bewegt sich nicht".<br /><br />Eine boolesche Variable kann mit dem Negationsoperator **invertiert** werden, z.B. die Variable play<br />`play = !play;`<br /><br />Eine boolesche Variable ist selbst ein boolescher Ausdruck, kann also allein in einer If-Bedingung stehen:<br />`if (play) { x++; }`
