# Find the Mistake

*Dateiname: find_the_mistake.md*

<img src="bilder/find_mistake_pic.jpg" width="500 px" />

Im Allgemeinen wird bei Programmen in zwei Fehlerarten unterschieden:

- syntaktische Fehler
- semantische Fehler


i>**Syntaktische Fehler** sind Zeichenfehler, welche zu Problemen beim Compilieren führen. Typische Beispiele sind vergessene Semikolon nach Befehlen oder falsch verwendete Datentypen.
Diese können mit Hilfe der händischen Quelltextfehlersuche herausgefunden werden. Processing gibt in einem solchen Falle eine Fehlermeldung aus.


i>**Semantische Fehler** sind logische Fehler im Algorithmus. Diese können mittels Debuggers oder eines händischen Schreibtischtests herausgefunden werden.
Wie dies genau geht, erfahrt ihr im folgenden Kapitel.

## Quelltextfehlersuche

Oftmals werden Fehler beim Compilieren angezeigt, an denen gar keine Fehler sind. Dies liegt daran, dass erst an der gemeldete Stelle der Fehler auffällt.

t>**Aufgabe**<br /> Finde alle 4 Fehler im gegebenen Quelltext

```
void setup() {
  size(400; 400);
}


void draw() {
  backgound(0);
  int x=0;
  while (x<width) {
    if (mouseX<1) {
      x+;
    } else {
      x=x+mouseX;
    
    fill(101);
    stroke(255);
    ellipse(x,150,16,16);
  }
}
```

t>**Aufgabe**<br />Finde die 3 Fehler.

```
float a;

void setup() {
  size(640, 360);
  stroke(255)
  a = height/2;
}

void draw() {
  background(51);
  line(0, a, width, a);  
  a == a - 0,5;
  if (a < 0) { 
    a = height; 
  }
}

```

## Schreibtischtest

Sollte das Ergebnis nicht wie erwartet sein, könnte ein sogenannter semantischer Fehler im Algorithmus vorliegen. Diese lassen sich händisch mittels eines Schreibtischtests herausfinden. Dazu notiert man alle verwendeten/relevanten Variablen und notiert deren zwischenzeitliche Belegung. Daraus wird oft schnell ersichtlich, an welcher Stelle des Algorithmus ein logischer Fehler implementiert wurde. Um dies zu üben, verwenden wir jedoch erst einmal Algorithmen, die korrekt sind.

```
int n=2;
int m=3;

int produkt;
if (m==0){print(1);}
if (m==1){print(n);}

produkt = n;
for(int i=2 ; i<=m ; i++){
  produkt = produkt*n;
}

print(produkt);
```

t>**Aufgabe**<br />Ermittle mit Hilfe eines Schreibtischtests die Funktion dieses Quelltextes bei Eingabewerten n=2 und m=3.

Zeile|n|m|produkt|Bedingung
-----|-|-|-------|---------
  3  | | |       |      


t>**Aufgabe**<br />Beschreibe die Ausgabe, die dieses Programm liefert. Teste mit unterschiedlichen Werten für a und b.

```
int a;
int b;
for (int i = 1; i <= 5; i++) { 
	a = 1;
	b = 1;
	for (int j = 7; j <= 10; j++) { 
		a = a + j;
		b = a * i; //Ausgabe a,b 
	}
} 
```


## Debugger

Der Debugger ist ein hilfsreiches Element, um semantische Fehler zu finden. Er führt das Programm nur bis zu sogenannten **Break-Points** aus. Alle beteiligten Variablen werden entsprechend aufgelistet und ihr bis dahin gespeicherter Wert angezeigt.

**Funktionsweise**

1. Menü->Debugger->Enable Debugger
2. Break-Points setzen, in dem du auf die Zeilennummer klickst (es erscheint ein Karo)
3. Sketch starten (Play-Symbol)

t>**Aufgabe**<br />Gibt es in diesem Programm einen Bug?

*Bug - Programmfehler oder Softwarefehler werden häufig Bug genannt*

*Tipp: Rechne das Beispiel vorher mit Hilfe der Tabellenkalkulation durch.*

```
double anfangsbetrag = 1000.0 ;
double dollars = 0.0;
double zinssatz ;
int jahre ;

zinssatz = 0.0 ;   // Zinssatz mit 0 beginnen

while ( dollars < 1000000 ){
   // Dollars nach 40 Jahren beim gegenwärtigen Zinssatz berechnen
   jahre   =  1 ;
   dollars = anfangsbetrag ;
   while (  jahre <= 40 )
     {
      dollars = dollars + dollars*zinssatz ;  // Zinsen für ein weiteres
                                              // Jahr addieren
      dollars = dollars + 1000 ;              // fügen Sie die jährliche
                                              // Einzahlung hinzu
      jahre    =  jahre + 1 ;
     }

     // zum nächsten Zinssatz wechseln
   zinssatz = zinssatz + 0.001 ;
}

println("Nach 40 Jahren bei " + zinssatz*100 + "Prozent Zinsen werden Sie" + dollars + " dollar haben." ) ;
```





