# Interaktion mit Maus und Tastatur

*Dateiname: interaktion.md*

---

![ComicArt Maus-Tastatur](bilder/maus-tastatur.png)

### Lernziele

?>Du kannst die Maus als Eingabewerkzeug verwenden.<br />
Du kannst bei den Mausinteraktionen zwischen einem "Gedrückt halten", "Klicken" und "Loslassen" unterscheiden und es zielgerichtet einsetzen.<br />
Du erkennst die Fehleranfälligkeit dieser Interaktionen.<br />
Du kennst Möglichkeiten der Tastaturabfrage, um sie in deinen Programmen zu verwenden.


---


## Interaktion mit der Maus

Interaktion bedeutet in unserer Lernumgebung ganz konkret, dass man nach dem Drücken von Start und vor dem Drücken von Stop Einfluss auf das Programm nehmen kann.
Processing stellt uns in diesem Kontext zwei einfache, aber durchaus praktische Platzhalter zur Verfügung, welche stets die aktuelle Mausposition an das Programm zurückgibt.
Probiere folgenden Code aus:

```
void setup() {
  size(300,300);
}

void draw() {
  ellipse(mouseX,mouseY,20,20);
}

```

Genial oder?


Wenn wir nun die Illusion wecken wollen, dass der Kreis sich bewegt und nicht stets neu gezeichnet wird, müssen wir unseren Quelltext nur ein wenig anpassen:

```
void setup() {
  size(300,300);
}

void draw() {
  background(200); //Bild löschen durch
                   //Übermalen
  ellipse(mouseX,mouseY,20,20);
}
```

Weitere Abwandlungen:

```
void setup() {
  strokeWeight(2);
}

void draw() {
  point(mouseX,mouseY);
}
```

```
void setup() {
  strokeWeight(2);
}

void draw() {
  background(255);
  line(pmouseX,pmouseY,mouseX,mouseY);
  //pmouseX ist die vorherige Maus-x-position
}
```

**Beispiel mit Anmerkungen**

```
void setup(){
  size(300,300);
  background(0);
  fill(255);
}
 
void draw(){
  background(0);
  fill(255);
  text("mouseX="+mouseX+" mouseY="+mouseY,0,12);
  text("frameRate="+(int)(frameRate),0,24);
  
  //prüft ständig, ob die Maustaste gedrückt ist
  if (mousePressed){
    if (mouseButton==LEFT){
      text("Linke Maustaste wird gehalten",0,36);
    }
  }
}
 
//Wird nur einmalig beim drauf klicken ausgeführt
void mousePressed(){
  if (mouseButton==RIGHT){
    fill(255);
    text("Rechte Maustaste wird gedrückt (Methode mousePressed)",0,48);
  }  
}
 
//Wird einmalig beim loslassen der Maustaste ausgeführt
void mouseClicked(){
  if (mouseButton==RIGHT){
    fill(0,255,0);
    text("Rechte Maustaste wird gedrückt (Methode mouseClicked)",0,60);
  }
}
```

Probier es aus!

---


## Übung

t>**Aufgabe 1 – Mauszeiger zeichnen**<br />Setze in der draw-Methode mit mouseX und mouseY eine Mausspur bestehend aus Punkten.

t>**Aufgabe 2 – Linien folgen dem Mauszeiger**<br />Die vier Linien, entstehend aus den Eckpunkten des Grafikfensters, sollen dem Mauszeiger folgen.

![Spinnennetz Beispielbild](bilder/beispiel-spinnennetz.png)

t>**Aufgabe 3 - Fadenkreuz**<br />Erzeuge ein kleines Fenster. In jenem soll ein Fadenkreuz, bestehend aus einer horizontalen und einer vertikalen Linie folgen.

![Fadenkreuz Beispielbild](bilder/beispiel-fadenkreuz.png)

t>**Aufgabe 4 - Linie und Kreis**<br />Zeichne eine Linie, die parallel zur y-Achse verläuft und immer der Maus folgt. Im Bereich zwischen dieser Linie und dem linken Rand soll mittig ein Kreis gezeichnet werden.

![Kreis-Linie Beispielbild](bilder/beispiel-maus-linie-kreis.png)

t>**Aufgabe 5 - Symmetrische Linie**<br />Programmiere eine Linie, deren eines Ende der Maus folgt, das andere Ende liegt immer punktgespiegelt auf der anderen Seite des Mittelpunkts.

t>**Aufgabe 6 - Mäusejagd**<br />An der aktuellen Mausposition soll der Schriftzug "maus" erscheinen (mittig über Maus, Schriftgröße 20).

[Hinweise zur Textgestaltung im Grafikfenster](https://processing.org/reference/text_.html)

t>**Aufgabe 7 - bewegtes Gesicht**<br />Aufgabe eines deiner ersten Grafikprogramme war das erstellen eines Gesichtes. Animiere die Augen so, dass sie innerhalb des weißen Bereiches bleiben. Jedoch sollen Sie dem Mauszeiger folgen.

![Kreis-Linie Beispielbild](bilder/beispiel-gesicht.png)

---

## Interaktion mit der Tastatur

Auch die Tastatur kann recht einfach zur Interaktion eingebunden werden.
Dies ist zum einen die Prozedur `void keyPressed(){…}` welche ausgeführt wird, sobald irgendeine Taste der Tastatur gedrückt wird.
Zum Anderen stehen die drei Variablen `keyPressed` (irgendeine Taste wird gedrückt - ist wahr oder falsch), `keyCode` (gibt den ASCII-Code der Taste zurück) und `key` (gibt die gedrückte Taste ans Programm zurück) zur Verfügung.
Auf der folgenden Seite sind einige Beispiele aufgeführt.


**Beispiel mit Anmerkungen**

```
void draw(){
  //es wird ständig geprüft, ob eine Taste gedrückt bzw. gehalten wird.
  if (keyPressed){
    println("Taste ist gedrückt");
  }
}
 
//wird auch bei Drücken einer Taste ausgeführt, 
//beim Halten der Taste ist diese Methode aber langsamer
void keyPressed(){
  print("key="+key);
  println(" keyCode="+keyCode);
}
```

```
void draw() {
  if (keyPressed) {
    if (key == 'b' || key == 'B') {
      fill(0);
    }
  } else {
    fill(255);
  }
  rect(25, 25, 50, 50);
}
```

Entsprechende Möglichkeiten brauchen wir im weiteren Verlauf des Unterrichts. Deswegen gibt es an dieser Stelle noch keine weiteren Übungsaufgaben.


?>Dieses Thema werden wir jedoch später nochmal aufgreifen.

---

## Hilfe

![Hilfe-Banner](bilder/hilfe.jpg)

>Aufgabe 1

Zeichne eine Ellipse in Abhängigkeit der aktuellen Mausposition mit `mouseX` und `mouseY`.

>Aufgabe 2

Um eine Animation ohne Spur zu erlangen, musst du im `draw()` stets das "Überzeichnen" mit Hilfe des `background()`-Befehls veranlassen.

>Aufgabe 3

Adaptiere (Übertrage) von der Aufgabe mit der Mauszeigerspur. die Linien befindet sich jeweils horizontal bzw. vertikal zur Mausposition, d.h. bei diesen Linien sind zwei Parameter jeweils `mouseX` bzw. `mouseY`.

>Aufgabe 4

keine Hilfe vorhanden

>Aufgabe 5

Hierfür gibt es mehrere Lösungen. Zeichne dir beispielhaft für eine beliebige Mausposition die entsprechenden Koordinaten der jeweiligen Endpunkte der Linie auf. Versuche diese Koordinaten in Abhängigkeit der Systemvariablen `mouseX` und `mouseY` zu bringen.

>Aufgabe 6

keine Hilfe vorhanden

>Aufgabe 7

Beachte, dass sich das Auge nicht aus dem "Augapfel" bewegen darf. Überlege dazu, wie groß der Wert von `mouseX` und `mouseY` werden kann. Dieser Wert muss durch eine entsprechende Zahl geteilt werden, damit der Kreis sich nur entsprechend nach links/rechts bewegen kann.
Ein kleines Beispiel.
Das Fenster ist 600 Pixel breit. Die Pupille ist 20 Pixel groß und der Augapfel 50 Pixel. Das heißt, die Pupille darf sich maximal 30 Pixel nach rechts/links bewegen. 600/(50-20) = 30. Weiter musst du jedoch beachten, dass die "Nullposition der Maus" nicht 0 sondern die Bildmitte (in unserem Fall 300) ist. Das heißt die Pupille muss noch um die Hälfte des errechneten Wertes versetzt werden.