# Statischer vs. aktiver Modus

*Dateiname: modus.md*

### Lernziele

?>Du entscheidest, ob für dein Programm der statische oder aktive Modus das richtige Instrument ist.<br />
Du verstehst, warum man beide Modi nicht mischen kann.<br />
Du kennst die Möglichkeit der Framerate-Einstellung.<br />

---

Bisher haben wir stets im sogenannten **statischen Modus** gearbeitet. Dabei ist weder Interaktion mit der Maus bzw. Tastatur oder Animation möglich.

Bringen wir doch etwas **Bewegung** ins Spiel. Und das ist in Processing gar nicht so schwierig. Jedoch muss man erst verstehen, in welche beiden Modi unterschieden wird.


i>**Statischer Modus**<br />In Processing können Befehle einfach nacheinander abgearbeitet werden. Dies geschieht zeilenweise und nennt man statischer Modus.


i>**Aktiver Modus**<br />Sobald wir Interaktion mit Maus oder Tastatur einbauen möchten, muss der aktive Modus verwendet werden. Dabei werden Grundeinstellungen im `setup()` definiert. Weiter gibt es einen sich dauerhaft wiederholenden Programmabschnitt, welcher im `draw()` definiert wird. Mehr Informationen dazu findest du in diesem Abschnitt.


## Aufbau

Wichtig hierbei zu wissen ist, dass beim aktiven Modus in einer vorgegebenen Wiederholungsrate (FrameRate) ein Bild(Pixelgrafik) erzeugt und dabei bestehende Pixel überschrieben werden.


Probier doch einfach mal folgenden Code aus:

```
void setup() {
  println(“und“);
}

void draw() {
  println(“hopp“);
}

```

Die Funktion `setup()` ist für die Starteinstellungen des Programms zuständig.

Die Funktion `draw()` wird in der eingestellten Wiederholungsrate unendlich oft aufgerufen und ermöglicht somit ein fortlaufendes Programm.

Sobald Processing eine davon erkennt, wird der **aktive Modus** ausgeführt.

Der Inhalt der beiden Funktionen steht im **Code-Block**, das ist der Bereich, der durch die jeweils nachstehenden geschweiften Klammern eingegrenzt ist.

s>Das Einrücken des Codes hilft bei der Übersicht des oftmals immer länger werdenden Quelltextes. Da nicht jeder so sorgsam mit dieser Option umgeht, hat Processing Abhilfe geschaffen. Durch das Drücke von **STRG-T** bzw. **CMD-T** wird der Code automatisch eingerückt - Super oder? ;-)

## Mischen Impossible ist untersagt!

Man darf aktiven und statischen Modus nicht mischen. Das heißt konkret, sobald `setup()` oder `draw()` erscheint, darf kein Code außerhalb des zugehörigen Code-Blocks geschrieben werden.


## Framerate

Die Frage ist, wie oft `draw()` denn nun wiederholt wird. In Anlehnung an Film bzw. Animation spricht man von frame rate, das Aufrufen/Abspielen von draw entspricht einem „frame“. Standardisiert ist die Wiederholungsrate auf 60 Mal pro Sekunde eingestellt. Diese Wiederholungsrate kann geändert werden:

```
void setup() {
  frameRate(5);
  println(“und“);
}

void draw() {
  println(“hopp“);
}

```
