# Bedingte Schleifen

*Dateiname: schleifen_bedingteSchleifen.md*

Im Alltag sind Schleifen (fachsprachlich: Zyklen) einer der wichtigsten Bestandteile, da gerade triviale Aufgaben mit sich **wiederholenden (Rechen)-Operationen** von Computern ausgeführt werden. Mit wenig Quelltext kann man recht viel „Output“ erzeugen.
Neben der Verzweigung ist ein Zyklus die zweite Möglichkeit, den *Kontrollfluss* des Programms zu steuern.

Durch die Kontrollflussänderung mittels einer Verzweigung wissen wir, dass der Computer nicht zwingend Zeile für Zeile abarbeiten muss, sondern auch Zeilen überspringen kann. Nun kannst du die Ausführung von Zeilen mehrfach **wiederholen**.

## While-Do-Schleife

Bilden wir die Analogie zur Verzweigung:

```
int zaehler = 0;

if (zaehler<3) {
  println("wow");
}
```

Hier wird ein Mal "wow" ausgegeben. Wenn du anstatt `if` `while` schreibst, wiederholt Processing den Code so lange, wie die Variable `zaehler` kleiner als 3 ist.

```
int zaehler = 0;

while (zaehler<3) {
  println("wow");
}
```

Da sich `zaehler` nicht ändert, läuft die Schleife **unendlich lang**. Nicht so gut!
Man nennt das eine Endlosschleife (entspricht damit nicht den Anforderungen eines Algorithmus). Und nebenbei auch ein beliebter Anfängerfehler. Die Lösung ist, `zaehler` bei jedem Durchlauf des Code-Blocks zu ändern.

```
int zaehler = 0;

while (zaehler<3) {
  println("wow");
  zaehler = zaehler + 1;
}
```

Jetzt wird der Code-Block drei Mal durchlaufen. Beim ersten Mal ist `zaehler` gleich 0, also wird der Block durchlaufen. Beim zweiten Mal ist `zaehler` gleich 1 - also wieder durch den Block.
Beim dritten Mal ist `zaehler` gleich 2 - also nochmal. Erst beim viertel Mal ist die Bedingung hinter `while` nicht mehr erfüllt und der Code-Block wird nicht mehr ausgeführt. Es geht weiter hinter der schließenden geschweiften Klammer.

Nach der While-Schleife geht es wieder regulär weiter im Code. Wir zeigen das, indem wir eine Anweisung anhängen. Dies wird immer ausgeführt:

```
int zaehler = 0;

while (zaehler<3) {
  println("wow");
  zaehler = zaehler + 1;
}
println("fertig");
```

## Schreibtischtest

Was ist bitte das?

Man kann sich die Verarbeitung einer Schleife gut mit Hilfe einer Tabelle verdeutlichen.

In der Tabelle trägt man die Entwicklung des Wertes der Laufvariablen ein (hier: zaehler). In einer weiteren Spalte sieht man, ob die Bedingung (zaehler < 3) für den jeweiligen Wert erfüllt ist oder nicht. In der letzten Spalte sehen wir die Schleifenaktion, die den Wert von zaehler erhöht.

Durchlauf | zaehler | zaehler < 3 | Aktion
--------- | ------- | ----------- | ------
1 | 0 | true | zaehler = 1
2 | 1 | true | zaehler = 2
3 | 2 | true | zaehler = 3
4 | 3 | false | - - -

Solche Schreibtischtests sind immer dann hilfreich, wenn die Entwicklung der Laufvariablen komplizierter wird (rückwärts, andere Schrittweite) oder wenn weitere Variablen in Abhängigkeit von der Laufvariablen gesetzt werden sollen.

w>Sollte mal in einem Algorithmus etwas schief gegangen sein und sucht den Fehler, bietet sich diese Methode idealerweise an.

### Darstellung der While-Do-Schleife

Änhlich wie bei der Verzweigung können wir den Kontrollfluss mit Hilfe eines Flussdiagramms visualisieren:

![Flussdiagramm While-Do](bilder/schleifen_whiledo.png)

Das Diagramm ist fast identisch mit dem der Verzweigung. Der einzige Unterschied ist, dass hier im true-Zweig ein Pfeil zurück zur Bedingung weist und damit Wiedehrolungen ermöglicht.

## Wichtige Begriffe

Die allgemeine Form der While-Schleife kann man so formulieren

```
while (BEDINGUNG)
  ANWEISUNG
```

i>Die Bedingung nennt man auch **Schleifenbedingung**. Nur wenn diese Bedingung erfüllt ist, wird der Code in der Schleife ausgeführt.<br /><br />Das Gegenteil der Schleifenbedingung (die logische Negation) nennt man übrigens **Abbruchbedingung**. In unserem Beispiel ist die Abbruchbedingung, dass `zaehler` größer gleich 3 ist. Ein häufiger Fehler ist, dass statt der Schleifenbedingung die Abbruchbedingung eingesetzt wird.<br /><br />Eine Variable, die hochgezählt wird und Teil der Schleifenbedingung ist, nennt man **Laufvariable** Die Tatsache, dass die Laufvariable hochgezählt wird, nennt man auch die **Schleifenaktion**. Diese ist besonders wichtig, weil man sie leicht vergißt, was wiederrum zur Folge hat, dass die Schleife nicht abbricht (Endlosschleife) und dein  Programm evtl. abstürzt.

i>Da die Bedingung für den Schleifendurchlauf zuerst steht (bzw. zuerst geprüft wird), nennt man diese Schleife auch **kopfgesteuerte Schleife**.

## Do-While-Schleife

Analog zur kopfgesteuerten Schleife, gibt es auch die **fußgesteuerte Schleife**. Sie ist wie folgt aufgebaut:

```
do {
  ANWEISUNG
} while (BEDINGUNG);
```

Der Unterschied zum vorher erläuterten Schleifentyp besteht darin, dass die Anweisung(en) nun mindestens einmal ausgeführt werden. Dies ist in manchen Berechnungen zwingenderweise nötig. Erst nach diesem Durchlauf wird geprüft, ob die Schleife weitergeführt wird.

