# Zählschleife

*Dateiname: schleifen_zaehlschleife.md*

Die Zählschleife ist die wohl am häufigsten verwendete Schleife. Bei ihr muss man die Anzahl der „Schleifendurchfläufe“ kennen.

Eine typisches Anwendungsbeispiel ist der sogenannte Wörterbuchangriff und gehört zu den sogenannten Brute-Force-Attacken. Es wird einfach eine Liste von möglichen Wörtern durchprobiert.

Die While-Schleifen sind schlicht und elegant, können aber zu Endlosschleifen führen, wenn man z.B. die Schleifenaktion vergißt. Eine weitere Beobachtung ist, dass fast immer eine Laufvariable eingeführt wird, die in der Schleifenbedingung genutzt und im Schleifenkörper verändert wird.

Da Informatiker von Natur aus faul sind, haben sie aus dieser Beobachtung heraus die For-Schleife (Zählschleife) erfunden. Die For-Schleife beitet ein kompaktes Wohlfühl-Paket für Schleifen mit Laufvariable mit relativ geringem Endlos-Risko.

```
for (int i=0; i<3; i=i+1){
  println("wow");
}
```

Ganz schön kurz, oder? Eine For-Schleife ist im Grunde eine Kurzschreibweise für die entsprechende While-Schleife:

```
int i=0; // 1. Initialisierung der Laufvariable

while(i<3){ // 2. Schleifenbedingung
  println("wow");
  i = i + 1; // 3. Schleifenaktion
}
```

Du siehst die **drei Komponenten** einer While-Schleife mit Laufvariable:
- Initialisierung,
- Schleifenbedingung und
- Schleifenaktion

Man kann im Codeblock auf die Laufvariable zugreifen (z.B. um sie auszugeben). Häufig wird die Laufvariable i genannt (englisch increment), aber es kann jeder beliebige Name verwendet werden.

In der Schleifenaktion wird meist der **Inkrementoperator** `++` verwendet, der die Laufvariable um 1 erhöht. Analog kann man den **Dekrementoperator** `--` verwenden.

w>Wichtig zu wissen ist, dass erst der Codeblock ausgeführt wird und als letzte Aktion die Veränderung der Laufvariable ausgeführt wird.

i>Die Laufvariable ist eine lokale Variable innerhalb der Zählschleife.

### Beispiele für Zählschleifen

```
for (int i = 10; i > 0; i--) {
  println(i);
}
```

```
int n = 5; // bis wohin gehen?

int summe = 0; // zum Aufsammeln

for (int i = 1; i <= n; i++) {
  summe = summe + i;
}

println(summe); // Ausgeben
```

i>**Historische Randnotiz**<br />Der Mathematiker **Carl Friedrich Gauß** entdeckte im zarten Alter von neun eine Formel, mit der sich diese Summe in einem Schritt berechnen lässt. Die Formel nennt man auch den **Kleinen Gauss**. Alle, die mit neun noch kein Mathe-Genie waren, müssen sich wohl mit der For-Schleife begnügen...