# Variablen

*Dateiname: variablen.md*

### Lernziele

?> Du lernst, wie man eine Variable verwendet.<br />
Du verstehst, was eine Variable im Arbeitsspeicher erzeugt.<br />
Du kennst Möglichkeiten für die Verwendung von komplexeren Variablennamen.

---

Variablen sind kleine Informationsspeicher. Sie können Daten sichern. Dazu muss Speicherplatz freigehalten werden.

**Anwendungsbeispiele**
- zentrale Stelle, wo häufig verwendete Werte abgelegt werden (bspw. Durchmesser von mehreren Kreisen)
- Wert während des Programmablaufs ändern zu können

i>Damit der Computer weiß, wie viel Speicherplatz bereit gestellt werden muss, ist zwingendermaßen der **Datentyp** der zu speichernden Information anzugeben. Weiter muss ein **Variablenname** für den Aufruf des Speicherinhalts definiert werden. Diesen Prozess nennt man **Deklaration einer Variablen**.

**Beispiel**
```
int zahl;
String name;
```

Allgemein sieht eine Deklaration wie folgt aus:

```
TYP NAME;
```

w>Variablennamen immer klein schreiben.<br />


## Zuweisung

i>Das erste Zuweisen eines Wertes zu einer Variablen wird als **Initialisierung** bezeichnet.

```
zahl = 5;
name = Manos;
```

Auf der rechten Seite kann ein beliebig komplexer arithmetischer Ausdruck stehen:
(nach bekannten Regeln: Punkt vor Strich, Klammerung)

```
zahl = (50*14) / 3 + 1;
```

w> Eine Zuweisung mit Gleichheitszeichen bedeutet nicht mathematische Gleichheit. Stattdessen wird zuerst die rechte Seite ausgewertet und dann das Resultat in die Variable geschrieben.

## Zuweisung mit Variablen

Variablen dürfen auch auf der rechten Seite stehen:

```
zahl2 = zahl1 + 15;
```

## Variablenwert verändern

Wie du später merken und anwenden wirst, kommt es sehr häufig vor, das der Wert einer Variable um einen bestimmten Betrag erhöht werden muss.

**Beispiel**

Variable `x` speichert eine x-Koordinate und hat den Wert 10, diesen möchtest du um 5 je draw-Durchlauf erhöhe

```
x = x + 5;
```

i>Bei der Veränderung "um 1 erhöhen" spricht man von **Inkrementieren**. Dafür gibt es eine Kurzschreibweise:

```
//statt
x = x + 1;
//kann man auch
x++;
//schreiben
```

i>Analog gibt es das **Dekrementieren**, was den Wert eine Variable um eins mindert. Die Kurzschreibweise sieht wie folgt aus:
```
x--;
```


## Globale Variablen

Wo steht die Deklaration im Code? Dazu beschäftigen wir uns mit sogenannten **globalen Variablen**. Alle diese Variablen sollten im *aktiven Modus* immer ganz oben im Code stehen.

```
// Alle Variablen ganz oben,
// also vor setup und draw
int x = 0;
int y = 0;

void setup() {}

void draw() {
   ellipse(x, y, 20, 20);
   x++;
   y++;
}
```

w>Globale variablen stehen im aktiven Modus immer ganz oben im Code, außerhalb von setup, draw und anderen Funktionen.

## Regeln bei der Arbeit mit Variablen

- `meineVariable` und `meinevariable` sind zwei unterschiedliche Variablen
  - Groß- und Kleinschreibung beachten
- dürfen nicht mit einem Schlüsselwort der Sprache Java identisch sein (bspw. `int` oder `float`)
- bestehen aus Ziffern und Buchstaben und zwei weiteren Zeichen (Unterstrich und $), die aber besser nicht verwendet werden
- dürfen nicht mit einer Ziffer beginnen
- dürfen keine Leerzeichen oder Bindestriche enthalten
- beginnen mit einem Kleinbuchstaben
- CamelBack-Regelung: meineNeueVariable
- sind möglichst aussagekräftig
