# Lokale Variablen

*Dateiname: variablen_lokal.md*

### Lernziele

?> Du verstehst den Unterschied zwischen einer globalen und einer lokalen Variablen.<br />
 Du verwendest passend zum Einsatzgebiet den richtigen Variablentypen.<br />
 Du verstehst das Verhalten des Arbeitsspeichers im Kontext von Variablen.
 
---

Bislang kennst du ausschließlich **globale Variablen**. Diese werden zu Beginn des Programms deklariert und können dann überall verwendet werden.

```
int x; // neue globale Variable

// Anfangsposition festlegen
void setup() {
  x = 0;
}

// Ball animieren
void draw() {
  background(255);
  ellipse(x, 50, 20, 20);
  x++;
}

// Ball zurücksetzen
void keyPressed() {
  x = 0;
}
```

Variable `x` nennt man eine globale Variable, weil überal, also global, im Programm verwendet werden kann. In unserem Beispiel wird `x` in drei verschiedenen Funktionen (setup, draw und keyPressed) verwendet.

## Skopus

Manchmal möchte man gar nicht, dass eine Variable überall verwendbar ist. Wie sich bei größeren Programmen herausstellt, wird der Code durch viele globale Variablen unübersichtlich. Daher gibt es **lokale Variablen**, die nur **eine eingeschränkte Lebensdauer** haben, d.h. sie sind nur in einem bestimmten Bereich gültig.

Den Bereich, innerhalb dessen eine Variable gültig ist, nennt man den **Skopus** der Variable (engl. scope).

Nehmen wir an, du möchtest einen Ball animieren. Der Durchmesser des Balls soll wachsen, also nehmen wir einfach den x-Wert auch als Durchmesser. Wir möchten aber, dass der Durchmesser nicht bei 0 sondern 10 beginnt. Also nehmen wir `x + 10` als Durchmesser:

```
float x = 0;

void draw() {
  ellipse(x, 50, x + 10, x + 10);
  x++;
}
```

Der Übersichtlichkeit willen möchten wir eine Variable für den Durchmesser einführen. Diese wird aber lediglich in draw() benötigt (nicht z.B. in setup). Daher definieren wir eine lokale Variable:

```
float x = 0;

void draw() {
  float durchmesser = x + 10; // Neue lokale Variable
  ellipse(x, 50, durchmesser, durchmesser);
  x++;
}
```

i>Eine **lokale Variable** ist nur im eigenen Code-Block gültig, angefangen von der Zeile, wo die Variable deklariert wird bis zur schließenden geschweiften Klammer des aktuellen Code-Blocks. Im oberen Beispiel hört die lokale Variable `durchmesser` auf zu leben, sobald `draw` abgearbeitet ist.

Willst du z.B. im folgenden Code auf die Variable `durchmesser` in der Funktion `setup()` zugreifen, dann bekommst du einen Fehler:

```
float x = 0;

void setup() {
  durchmesser = 10; // FEHLER! VARIABLE UNBEKANNT
}

void draw() {
  float durchmesser = x + 10; // Neue lokale Variable
  ellipse(x, 50, durchmesser, durchmesser);
  x++;
}
```

s>Doch lokale Variablen haben entscheidende **Vorteile**. Erstens belegen sie nicht dauerhaft den reservierten Speicherplatz (**speicherarm**). Weiter können Sie nicht von einer externen Funktion fälschlicherweise beeinflusst werden.

---

## Übung

<iframe src="https://learningapps.org/watch?app=2542257" style="border:0px;width:100%;height:500px" webkitallowfullscreen="true" mozallowfullscreen="true"></iframe>


t>**Aufgabe 1 - böser Zwilling**<br />Programmiere einen weißen Kreis, der dem Mauszeiger folgt. Gleichzeitig soll sein schwarzer Zwilling eingeblendet werden, der die Bewegung des weißes Kreises punktsymmetrisch zum Mittelpunkt des Fensters vollzieht.

t>**Aufgabe 2 - Mitte**<br />Programmiere einen weißen Kreis, der dem Mauszeiger folgt und mit der linken oberen Ecke des Fensters durch eine Linie verbunden ist. In der Hälfte der Linie soll ein zweiter Kreis dargestellt werden.

t>**Aufgabe 3 - World in Miniature**<br />Programmiere ein kleines Rechteck in der Ecke. Der Punkt dort zeigt die Position der Maus im Gesamtfenster an. Du kannst folgenden Code als Ausgangspunkt nutzen:

<img src="bilder/world-of-miniature.png" width="350px" />

```
// Position/Größe des Rechtecks
float x = 530;
float y = 350;
float w = 60;
float h = 40;

void setup() {
  size(600, 400);
}

void draw() {
  background(0);
  stroke(255);
  noFill();
  rect(x, y, w, h);
  fill(255);
  ellipse(mouseX, mouseY, 10, 10); // Mauszeiger

  // Dein Code ...
}
```

t>**Aufgabe 3b**<br />Erweiter dein Programm so, dass du gänzlich ohne lokale Variablen auskommst. Dazu muss sich die Miniaturwelt jedoch stets skaliert zum Fenster verhalten.