# 3D-Objekte in Processing

*Dateiname: vertiefung_3d.md*

### Allgemeines

Natürlich ist es in Processing auch möglich in 3D zu arbeiten. Neben der x- und y-Achse wird das Koordinatensystem um die z-Achse erweitert.

## Grundbefehle

Befehl|Erläuterung
------|-----------
size(600,400,P3D)|3D-Render aktivieren
translate()|Methode, ermöglicht die Zuweisung des Betrachters im 3-Dimensionalen (siehe Beispiel 1)
pushMatrix()|Speichert die Position des Koordinatensystems
popMatrix()|Stellt das mit pushMatrix() gespeicherte Koordinatensystem wieder her
scale(WERT), scale(WERTx,WERTy)|Größe des Objektes kann geändert werden. Der Wert steht für einen Multiplikator
rotate(WINKEL),rotateX(WINKEL),rotateY(WINKEL),rotateZ(WINKEL)|Drehen des Koordinatensystems möglich
camera(eyeX,eyeY,eyeZ,centerX,centerY,centerZ,upX,upY,upZ)|Methode, ermöglicht das Einstellen einer virtuellen Kamera
perspective()|Methode, ermöglicht die Einstellung der Brennweite


## 3D spezifische Elemente (Körper)

Befehl|Erläuterung
------|-----------
sphare(int size) | erzeugt eine Kugel
sphareDetail(int detail) | bestimmt die Auflösung der Textur (detail=40 bedeutet 360/40, d.h. alle 9 Grad wird ein Punkt gezeichnet)
box(int seitenlänge)| erzeugt einen Würfel

### Komplexe Formen
Sollten komplexere Formen benötigt werden, kann man diese entweder über beginShape(), vertex(x,y,z) und endShape() in Processing erstellen

```
//3-seitige Pyramide
  beginShape(TRIANGLE);
  vertex(-1, -1, 1);
  vertex( -1, 1, 1);
  vertex( 0, 0, -1);

  vertex(-1, 1, 1);
  vertex( 1, 0, 1);
  vertex( 0, 0, -1);

  vertex(-1, -1, 1);
  vertex( 1, 0, 1);
  vertex( 0, 0, -1);

  vertex(-1, -1, 1);
  vertex( 1, 0, 1);
  vertex( -1, 1, 1);

  endShape();
```

### Lichtquellen

Generell müssen Lichtquellen in jedem draw() und nicht setup() durchlaufen werden

Befehl|Erläuterung
------|-----------
light()|generiert Kombinationen aus verschiedenen Lichtquellen
ambientLight(r,g,b)|dient als Hintergrundbeleuchtung
directionalLight(r,g,b,x,y,z)|erzeugt paralleles Licht aus einer bestimmten Richtung (x,y,z)
lightSpecular(r,g,b)|legt die Farbe der Glanzlichter fest. Hat nur Einfluss auf diejenigen Elemente, die im Code danach erstellt werden

### Oberflächen

Die Effekte von Lichtquellen hängen von der Oberflächenbeschaffenheit der beleuchteten Objekte ab. Deshalb gibt es in Processing eine Reihe von Oberflächeneigenschaften, die in Kombination mit den verwendeten Lichtquellen benutzt werden können.

`ambient(r,g,b)`

Legt den Reflexionsgrad der Oberfläche in Kombination mit ambientLight() fest. z.B. ambient(255,128,0) reflektiert rot zur Gänze, grün zur Hälfte und blau gar nicht.

![Bild Lichtquelle](bilder/3d_ambient.png)

`emissive(r,g,b)`

Objekte können selbst Licht emittieren. D.h. sie leuchten selbst in der hier definierten Farbe. Alledings nur dann, wenn irgendeine Lichtquelle vorhanden ist.

![Bild Lichtquelle](bilder/3d_emissive.png)

`specular(r,g,b)`

Damit definiert man die Farbe der Glanzlichter.

![Bild Lichtquelle](bilder/3d_specular.png)

`shininess(float)`

Legt die Glanzintensität einer Oberfläche fest.

![Bild Lichtquelle](bilder/3d_shininess.png)

`lightFalloff(constant, linear, quadratic)`

Dieser Parameter legt den Lichtabfall über die Entfernung zur Lichtquelle mit Werten zwischen 0 und 1 fest. Er wirkt allerdings nur auf point spot und ambient Lights.




## Beispiele

### translate, rotate

```
void setup() {
  size(300, 300);
  background(0);
  stroke(255, 20);
  frameRate(5); }

void draw() {
  translate(150, 150);
  strokeWeight(frameCount/4);
  rotate(radians(frameCount*10));
  line(0, 0, 100, 0); }

```

### Rotation einer „verknüpften“ Fläche (vertex)

```
float angle = 0;

void setup() {
  size(130, 130, OPENGL);
}

void draw() {
  background(100);

  translate(60, 40, 0);
  rotateY(angle);
  angle += .01;

  beginShape();
  vertex(0, 0, 0);
  vertex(50, 0, 0);
  vertex(50, 50, 0);
  vertex(0, 50, 0);
  endShape(CLOSE);
}

```

### 3D-Objekte

```
size(640, 360, P3D);
background(0);
lights();

pushMatrix();
translate(130, height/2, 0);
rotateY(1.25);
rotateX(-0.4);
noStroke();
box(100);
popMatrix();

pushMatrix();
translate(500, height*0.35, -200);
noFill();
stroke(255);
sphere(280);
popMatrix();

```

### Drehbares Koordinatensystem

```
void setup() {
  size(640, 640, P3D);
}
void draw() {
  background(0);
  textSize(20);
  stroke(255);
  translate(width/2, height/2, -200);
  println((height/2.0) / tan(PI*60.0 / 360.0));
  rotateX(map(mouseY, 0, height, -PI, PI));
  rotateY(map(mouseX, 0, height, -PI, PI));
  drawAxes(); }

void drawAxes() {
  stroke(255, 0, 0);
  line(-300, 0, 0, 300, 0, 0);
  text("+x", 300, 0, 0);
  text("-x", -330, 0, 0);
  stroke(0, 255, 0);
  line(0, -300, 0, 0, 300, 0);
  text("+y", 0, 330, 0);
  text("-y", 0, -300, 0);
  stroke(0, 0, 255);
  line(0, 0, -300, 0, 0, 300);
  text("+z", 0, 0, 330);
  text("-z", 0, 0, -300);
}
```

### Drehbare Kugel

```
void setup() {
  size(640, 640, P3D);
  sphereDetail(40);
}
void draw() {
  background(0);
  translate(width/2, height/2, -200);
  rotateX(map(mouseY, 0, height, -PI, PI));
  rotateY(map(mouseX, 0, height, -PI, PI));
  drawAxes();
  fill(255, 40);
  sphere(200);
}

void drawAxes() {
  textSize(20);
  fill(255);
  stroke(255, 0, 0);
  line(-300, 0, 0, 300, 0, 0);
  text("+x", 300, 0, 0);
  text("-x", -330, 0, 0);
  stroke(0, 255, 0);
  line(0, -300, 0, 0, 300, 0);
  text("+y", 0, 330, 0);
  text("-y", 0, -300, 0);
  stroke(0, 0, 255);
  line(0, 0, -300, 0, 0, 300);
  text("+z", 0, 0, 330);
  text("-z", 0, 0, -300);
}
```

### 3-seitige Pyramide

```
void setup() {
  size(640, 640, P3D);
}
void draw() {

  background(50);
  textSize(20);
  noStroke();
  translate(width/2, height/2, -200);
  rotateX(map(mouseY, 0, height, -PI, PI));
  rotateY(map(mouseX, 0, height, -PI, PI));
  drawAxes();
  scale(100);

  //Zeichne 3-seitige Pyramide
  beginShape(TRIANGLE);
  vertex(-1, -1, 1);
  vertex( -1, 1, 1);
  vertex( 0, 0, -1);

  vertex(-1, 1, 1);
  vertex( 1, 0, 1);
  vertex( 0, 0, -1);

  vertex(-1, -1, 1);
  vertex( 1, 0, 1);
  vertex( 0, 0, -1);

  vertex(-1, -1, 1);
  vertex( 1, 0, 1);
  vertex( -1, 1, 1);

  endShape();
}

void drawAxes() {
  stroke(255, 0, 0);
  line(-300, 0, 0, 300, 0, 0);
  text("+x", 300, 0, 0);
  text("-x", -330, 0, 0);
  stroke(0, 255, 0);
  line(0, -300, 0, 0, 300, 0);
  text("+y", 0, 330, 0);
  text("-y", 0, -300, 0);
  stroke(0, 0, 255);
  line(0, 0, -300, 0, 0, 300);
  text("+z", 0, 0, 330);
  text("-z", 0, 0, -300);
}
```

### Kamera-Position

```
void setup() {
  size(200, 200, OPENGL);
}

void draw() {
  background(100);

  float eyeX = mouseX;
  float eyeY = mouseY;

  camera(eyeX, eyeY, (height/2.0) / tan(PI*30.0 / 180.0), 
    width/2.0, height/2.0, 0, 
    0, 1, 0);

  translate(100, 100, 0);
  box(80, 50, 30);
}

```


## Weiterführende Links

- [Michael Kipp 3D](https://michaelkipp.de/processing/15%20grafik3d.html)
- [https://lernprocessing.wordpress.com](https://lernprocessing.wordpress.com/2012/04/17/3-d-basics/)
- [Processing 3D](https://processing.org/tutorials/p3d/)
- [http://www.gm.fh-koeln.de/](http://www.gm.fh-koeln.de/~konen/WPF-Spiele/materialien/Tut-Processing-P3D.pdf)